import numpy as np
from numpy import ndarray

PEAK = 1
VALLEY = -1

def identify_initial_pivot(data, up_thresh, down_thresh):
    
    initial_value = data[0]
    next_value = initial_value

    previous_maximum_value = initial_value
    previous_minimum_value = initial_value

    max_t = 0
    min_t = 0

    up_thresh += 1
    down_thresh += 1

    for t in range(1, len(data)):
        next_value = data[t]

        if next_value / previous_minimum_value >= up_thresh:
            return VALLEY if min_t == 0 else PEAK

        if next_value / previous_maximum_value <= down_thresh:
            return PEAK if max_t == 0 else VALLEY

        if next_value > previous_maximum_value:
            previous_maximum_value = next_value
            max_t = t

        if next_value < previous_minimum_value:
            previous_minimum_value = next_value
            min_t = t

    t_n = len(data)-1
    return VALLEY if initial_value < data[t_n] else PEAK


def peak_valley_pivots(open_data, data, up_thresh=0.1, down_thresh=-0.1, deviation_thresh=5, depth=5):
    """
    Find the peaks and valleys of a series.
    :param open_data: Open series 
    :param data: the Close series to analyze
    :param up_thresh: minimum relative change necessary to define a peak
    :param down_thesh: minimum relative change necessary to define a valley
    :param deviation_thresh: minimum percentage required to define a peak or valley
    :param depth: minimum number of candles between peak and valley
    :return: an array with 0 indicating no pivot and -1 and 1 indicating
        valley and peak
    The First and Last Elements
    ---------------------------
    The first and last elements are guaranteed to be annotated as peak or
    valley even if the segments formed do not have the necessary relative
    changes. This is a tradeoff between technical correctness and the
    propensity to make mistakes in data analysis. The possible mistake is
    ignoring data outside the fully realized segments, which may bias
    analysis.
    """

    if down_thresh > 0:
        raise ValueError('The down_thresh must be negative.')

    initial_pivot = identify_initial_pivot(data, up_thresh, down_thresh)
    t_n  = len(data)
    pivots = np.zeros(t_n, dtype=np.int_)
    trend = -initial_pivot
    last_pivot_t = 0
    last_pivot_x = data[0]

    pivots[0] = initial_pivot
    
    last_peak_index = 0
    last_valley_index = 0

    # Adding one to the relative change thresholds saves operations. Instead
    # of computing relative change at each point as x_j / x_i - 1, it is
    # computed as x_j / x_1. Then, this value is compared to the threshold + 1.
    # This saves (t_n - 1) subtractions.
    up_thresh += 1
    down_thresh += 1
#     print("Up thresh: ", up_thresh)
#     print("Down thresh: ", down_thresh)
#     up_thresh = 
#     down_thresh = 0.2

    for t in range(1, t_n):
        x = data[t]
        r = x / last_pivot_x
        
#         if t == 12322:
#             print("x: ", x)
#             print("trend: ", trend)
#             print("last peak: ", last_peak_index )
#             print("Last pivot x: ", last_pivot_x)
        
#         print("Last pivot t: ", last_pivot_t)
        if trend == -1:
#             print("Valley: ", r)
            if r >= up_thresh:
                current_index = last_pivot_t
                deviation = (abs(data[current_index]  - data[last_peak_index])/(data[last_peak_index])) * 100

                if deviation > deviation_thresh:
                    if (current_index - last_peak_index) >= depth:
                        # Valley should be lower than the peak
                        if data[current_index] < data[last_peak_index]:
                            
                            # If candle is red
                            if open_data[last_pivot_t] > data[last_pivot_t]:
                                pivots[last_pivot_t] = trend
                                # print("="*30)
                                # print("% change from last peak: ", (abs(data[current_index] - data[last_peak_index])/(data[last_peak_index])) * 100)
                                # print("Current Index: ", last_pivot_t)
                                # print("Previous peak index: ", last_peak_index)
    #     #                         print("Difference: ", (current_index - last_peak_index))
                                # print("Valley detected at: ", data[last_pivot_t])

                                last_valley_index = current_index
                                trend = PEAK
                                last_pivot_x = x
                                last_pivot_t = t
                            else:
                                last_pivot_x = x
                                last_pivot_t = t                                
                        else:
                            last_pivot_x = x
                            last_pivot_t = t
                    else:
                        last_pivot_x = x
                        last_pivot_t = t
                
                elif last_pivot_x >= data[last_peak_index]:
                    # If candle is green
                    if open_data[last_pivot_t] < data[last_pivot_t]:
                        # print("="*30)
                        # print("Setting last peak index in deviation : ", last_peak_index, " to: ", last_pivot_t)
                        pivots[last_peak_index] = 0
                        pivots[last_pivot_t] = PEAK
                        last_peak_index = last_pivot_t
                        last_pivot_x = x
                        last_pivot_t = t
                    else:
                        last_pivot_x = x
                        last_pivot_t = t
                    
                else:
                    last_pivot_x = x
                    last_pivot_t = t
            
            elif last_pivot_x >= data[last_peak_index]:
                # If candle is green
                if open_data[last_pivot_t] < data[last_pivot_t]:
                    # print("="*30)
                    # print("Setting last peak index : ", last_peak_index, " to: ", last_pivot_t)
                    pivots[last_peak_index] = 0
                    pivots[last_pivot_t] = PEAK
                    last_peak_index = last_pivot_t
                    last_pivot_x = x
                    last_pivot_t = t
                else:
                    last_pivot_x = x
                    last_pivot_t = t
                
                    
            elif x < last_pivot_x:
                last_pivot_x = x
                last_pivot_t = t

        else:
#             print("Peak: ", r)
            if r <= down_thresh:
#                 print("="*30)
#                 print("Current x: ", x)
#                 print("Last pivot x: ", last_pivot_x)
                current_index = last_pivot_t
                deviation = (abs(data[current_index] - data[last_valley_index])/(data[last_valley_index])) * 100
                if deviation > deviation_thresh:

                    if (current_index - last_valley_index) >= depth:
                        if data[current_index] > data[last_valley_index]:
                            
                            # If candle is green
                            if open_data[last_pivot_t] < data[last_pivot_t]:
                                pivots[last_pivot_t] = trend
                                # print("="*30)
                                # print("% change from last valley: ", (abs(data[current_index] - data[last_valley_index])/(data[last_valley_index])) * 100)
                                # print("Current Index: ", last_pivot_t)
                                # print("Previous valley index: ", last_valley_index)
    #     #                         print("Difference: ", (current_index - last_valley_index))
                                # print("Peak detected at: ", data[last_pivot_t])
                                last_peak_index = current_index
                                trend = VALLEY
                                last_pivot_x = x
                                last_pivot_t = t
                            else:
                                last_pivot_x = x
                                last_pivot_t = t                                 
                        else:
                            last_pivot_x = x
                            last_pivot_t = t               
                    else:
                        last_pivot_x = x
                        last_pivot_t = t
                
                elif last_pivot_x <= data[last_valley_index]:
#                     if t == 14253:
#                         print("Close value: ", data[t])
#                         print("Last valley index: ", last_valley_index)
#                         print("Last valley value: ", data[last_valley_index])
                    # If candle is red
                    if open_data[last_pivot_t] > data[last_pivot_t]:
                        # print("="*30)
                        # print("Setting last valley index in deviation : ", last_valley_index, " to: ", last_pivot_t)
                        pivots[last_valley_index] = 0
                        pivots[last_pivot_t] = VALLEY
                        last_valley_index = last_pivot_t
                        last_pivot_x = x
                        last_pivot_t = t
                    
                    else:
                        last_pivot_x = x
                        last_pivot_t = t
                        
                else:
                    last_pivot_x = x
                    last_pivot_t = t
            
            elif last_pivot_x <= data[last_valley_index]:
                # If candle is red
                if open_data[last_pivot_t] > data[last_pivot_t]:
                    # print("="*30)
                    # print("Setting last valley index : ", last_valley_index, " to: ", last_pivot_t)
                    pivots[last_valley_index] = 0
                    pivots[last_pivot_t] = VALLEY
                    last_valley_index = last_pivot_t
                    last_pivot_x = x
                    last_pivot_t = t
                else:
                    last_pivot_x = x
                    last_pivot_t = t
                
            elif x > last_pivot_x:
                last_pivot_x = x
                last_pivot_t = t

#     if last_pivot_t == t_n-1:
#         pivots[last_pivot_t] = trend
#     elif pivots[t_n-1] == 0:
#         pivots[t_n-1] = -trend
        

    return pivots
